package com.baidu.cloud.videoplayer.demo.db;

import android.content.ContentUris;
import android.content.ContentValues;
import android.database.Cursor;
import android.net.Uri;

import com.baidu.cloud.videoplayer.demo.BaseApp;

import java.util.ArrayList;
import java.util.List;

public class DBManager {
    private volatile static DBManager instance;

    public static DBManager getInstance() {
        if (instance == null) {
            synchronized (DBManager.class) {
                if (instance == null) {
                    instance = new DBManager();
                }
            }
        }
        return instance;
    }

    /**
     * insert a new record with the path of media
     *
     * @param media path of media must be specified
     */
    private void insertHistory(MediaBean media) {
        ContentValues values = new ContentValues();
        values.put(MediaBean.BEAN_PATH, media.path);
        values.put(MediaBean.BEAN_POSITION, media.position);
        values.put(MediaBean.BEAN_DURATION, media.duration);
        Uri uri = BaseApp.getAppContext().getContentResolver().insert(MediaProvider.HISTORY_CONTENT_URI, values);
        media.id = ContentUris.parseId(uri);
    }

    /**
     * query all record from history table
     *
     * @return count of records
     */
    public List<MediaBean> queryHistory() {
        List<MediaBean> medias = new ArrayList<>();
        Cursor cursor = BaseApp.getAppContext().getContentResolver().query(MediaProvider.HISTORY_CONTENT_URI,
                null, null, null, null);
        if (cursor == null) {
            return medias;
        }
        if (cursor.getCount() == 0) {
            cursor.close();
            return medias;
        }
        int idIndex = cursor.getColumnIndexOrThrow(MediaBean.BEAN_ID);
        int dataIndex = cursor.getColumnIndexOrThrow(MediaBean.BEAN_PATH);
        int positionIndex = cursor.getColumnIndexOrThrow(MediaBean.BEAN_POSITION);
        int durationIndex = cursor.getColumnIndexOrThrow(MediaBean.BEAN_DURATION);
        cursor.moveToFirst();
        int count = cursor.getCount();
        for (int i = 0; i < count; i++) {
            MediaBean media = new MediaBean();
            media.id = cursor.getLong(idIndex);
            media.path = cursor.getString(dataIndex);
            media.position = cursor.getInt(positionIndex);
            media.duration = cursor.getInt(durationIndex);
            medias.add(media);
            cursor.moveToNext();
        }
        cursor.close();
        return medias;
    }

    /**
     * update if one record exists, otherwise insert it
     *
     */
    public void updateHistory(MediaBean media) {
        ContentValues values = new ContentValues();
        values.put(MediaBean.BEAN_POSITION, media.position);
        values.put(MediaBean.BEAN_DURATION, media.duration);
        values.put(MediaBean.BEAN_PATH, media.path);
        String where = MediaBean.BEAN_ID + " = ?";
        String[] selectionArgs = {
                media.id + ""
        };
        int count = BaseApp.getAppContext().getContentResolver().update(MediaProvider.HISTORY_CONTENT_URI,
                values, where, selectionArgs);
        if (count == 0) {
            insertHistory(media);
        }
    }
}
