package com.baidu.cloud.videoplayer.demo.db;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.util.Log;

import androidx.annotation.NonNull;

public class MediaProvider extends ContentProvider {

    private static final String TAG = "MediaProvider";
    private static final int DATABASE_VERSION = 2;
    private static final String DATABASE_NAME = "media.db";

    private static final String AUTOHORITY = "com.baidu.cloud.videoplayer.demo.db";
    public static final String TABLE_HISTORY = "history";
    public static final Uri HISTORY_CONTENT_URI = Uri
            .parse("content://" + AUTOHORITY + "/" + TABLE_HISTORY);

    private static final int HISTORY_TABLE = 1;
    private static final int HISTORY_ROW = 2;
    private static final String HISTORY_TYPE = "vnd.android.cursor.dir/com.baidu.cloud.videoplayer.demo.history";
    private static final String HISTORY_ITEM_TYPE = "vnd.android.cursor.item/com.baidu.cloud.videoplayer.demo.history";

    private static final UriMatcher mUriMatcher;
    static {
        mUriMatcher = new UriMatcher(UriMatcher.NO_MATCH);
        mUriMatcher.addURI(AUTOHORITY, TABLE_HISTORY, HISTORY_TABLE);
        mUriMatcher.addURI(AUTOHORITY, TABLE_HISTORY + "/#", HISTORY_ROW);
    }

    private SQLiteDatabase mDatabase;

    @Override
    public int delete(@NonNull Uri uri, String selection, String[] selectionArgs) {
        int count;
        String id;
        String where;
        switch (mUriMatcher.match(uri)) {
            case HISTORY_TABLE:
                count = mDatabase
                        .delete(TABLE_HISTORY, selection, selectionArgs);
                break;
            case HISTORY_ROW:
                id = uri.getPathSegments().get(1);
                where = MediaBean.BEAN_ID + " = " + id;
                count = mDatabase.delete(TABLE_HISTORY, where, selectionArgs);
                break;
            default:
                Log.e(TAG, "delete failed");
                return 0;
        }
        getContext().getContentResolver().notifyChange(uri, null);
        return count;
    }

    @Override
    public String getType(@NonNull Uri uri) {
        switch (mUriMatcher.match(uri)) {
            case HISTORY_TABLE:
                return HISTORY_TYPE;
            case HISTORY_ROW:
                return HISTORY_ITEM_TYPE;
            default:
                Log.e(TAG, "getType failed");
                return null;
        }
    }

    @Override
    public Uri insert(@NonNull Uri uri, ContentValues values) {
        long rowId;
        if (mUriMatcher.match(uri) == HISTORY_TABLE) {
            rowId = mDatabase.insert(TABLE_HISTORY, null, values);
        } else {
            Log.e(TAG, "insert failed, uri: " + uri + " is not matched");
            return null;
        }
        if (rowId > 0) {
            Uri newUri = ContentUris.withAppendedId(HISTORY_CONTENT_URI, rowId);
            getContext().getContentResolver().notifyChange(newUri, null);
            return newUri;
        }
        Log.e(TAG, "insert failed");
        return null;
    }

    @Override
    public boolean onCreate() {
        Context context = getContext();
        SQLiteHelper helper = new SQLiteHelper(context, DATABASE_NAME, null, DATABASE_VERSION);
        mDatabase = helper.getWritableDatabase();
        return mDatabase != null;
    }

    @Override
    public Cursor query(@NonNull Uri uri, String[] projection, String selection, String[] selectionArgs,
                        String sortOrder) {
        Cursor cursor;
        String id;
        String where;
        switch (mUriMatcher.match(uri)) {
            case HISTORY_TABLE:
                cursor = mDatabase.query(TABLE_HISTORY, null, selection,
                        selectionArgs, null, null, sortOrder);
                return cursor;
            case HISTORY_ROW:
                id = uri.getPathSegments().get(1);
                where = MediaBean.BEAN_ID + " = " + id;
                cursor = mDatabase.query(TABLE_HISTORY, null, where,
                        selectionArgs, null, null, sortOrder);
                return cursor;
            default:
                Log.e(TAG, "query failed, uri: " + uri + " is not matched");
                return null;
        }
    }

    @Override
    public int update(@NonNull Uri uri, ContentValues values, String selection, String[] selectionArgs) {
        int count;
        String id;
        String where;
        switch (mUriMatcher.match(uri)) {
            case HISTORY_TABLE:
                count = mDatabase.update(TABLE_HISTORY, values, selection,
                        selectionArgs);
                break;
            case HISTORY_ROW:
                id = uri.getPathSegments().get(1);
                where = MediaBean.BEAN_ID + " = " + id;
                count = mDatabase.update(TABLE_HISTORY, values, where,
                        selectionArgs);
                break;
            default:
                Log.e(TAG,"update failed, uri: " + uri + " is not matched");
                return 0;
        }
        if (count > 0) {
            getContext().getContentResolver().notifyChange(uri, null);
        }
        return count;
    }
}
