package com.baidu.cloud.videoplayer.demo.popview;

import com.baidu.cloud.media.player.feature.FeatureManager;
import com.baidu.cloud.videoplayer.demo.BaseApp;
import com.baidu.cloud.videoplayer.demo.C;
import com.baidu.cloud.videoplayer.demo.SampleListActivity;
import com.baidu.cloud.videoplayer.demo.R;
import com.baidu.cloud.videoplayer.demo.helper.CommonUtils;
import com.baidu.cloud.videoplayer.demo.helper.HdrKitWrapper;
import com.baidu.cloud.videoplayer.demo.info.SharedPrefsStore;

import android.content.Context;
import android.graphics.drawable.ColorDrawable;
import android.util.DisplayMetrics;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.CompoundButton;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.PopupWindow;
import android.widget.RadioButton;
import android.widget.Toast;

public class SettingsPopWindow extends PopupWindow {
    private View contentView;
    private SampleListActivity activity;
    private DisplayMetrics dm = new DisplayMetrics();

    public SettingsPopWindow(final SampleListActivity context, View ctView) {
        super(ctView, LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT, true);
        contentView = ctView;
        activity = context;
        context.getWindowManager().getDefaultDisplay().getMetrics(dm);
        // 设置SelectPicPopupWindow的View
//        this.setContentView(contentView);
//        // // 设置SelectPicPopupWindow弹出窗体的宽
//        this.setWidth(LayoutParams.WRAP_CONTENT);
//        // // 设置SelectPicPopupWindow弹出窗体的高
//        this.setHeight(LayoutParams.WRAP_CONTENT);
//        // 设置SelectPicPopupWindow弹出窗体可点击
//        this.setFocusable(true);
        this.setOutsideTouchable(true);
        // 刷新状态
        this.update();
        // 实例化一个ColorDrawable颜色为半透明
        ColorDrawable dw = new ColorDrawable(0x000000);
        // 点back键和其他地方使其消失,设置了这个才能触发OnDismisslistener ，设置其他控件变化等操作
        this.setBackgroundDrawable(dw);
        this.setAnimationStyle(android.R.style.Animation_Dialog);
        final RadioButton rbHori = contentView.findViewById(R.id.rb_setting_hori);
        final RadioButton rbVerti = contentView.findViewById(R.id.rb_setting_verti);
        int orientType = SharedPrefsStore.getIntData(context, C.KEY_ORIENT_TYPE, C.ORIENT_PORTRAIT);
        if (orientType == C.ORIENT_PORTRAIT) {
            rbVerti.setChecked(true);
        } else {
            rbHori.setChecked(true);
        }

        final RadioButton rbSimple = contentView.findViewById(R.id.rb_setting_controlbar_simple);
        final RadioButton rbAdvance = contentView.findViewById(R.id.rb_setting_controlbar_advanced);
        int barType = SharedPrefsStore.getIntData(context, C.KEY_CONTROLBAR_TYPE, C.BARTYPE_ADVANCE);
        if (barType == C.BARTYPE_SIMPLE) {
            rbSimple.setChecked(true);
        } else {
            rbAdvance.setChecked(true);
        }

        final RadioButton rbAutoResumeOn = contentView.findViewById(R.id.rb_setting_autoresume_on);
        final RadioButton rbAutoResumeOff = contentView.findViewById(R.id.rb_setting_autoresume_off);
        boolean autoResumeSwitch = SharedPrefsStore.getBooleanData(context, C.KEY_AUTORESUME_SWITCH);
        if (autoResumeSwitch == C.AUTORESUME_ON) {
            rbAutoResumeOn.setChecked(true);
        } else {
            rbAutoResumeOff.setChecked(true);
        }

        final RadioButton rbLoopPlayOn = contentView.findViewById(R.id.rb_setting_loopplay_on);
        final RadioButton rbLoopPlayOff = contentView.findViewById(R.id.rb_setting_loopplay_off);
        boolean loopPlaySwitch = SharedPrefsStore.getBooleanData(context, C.KEY_LOOPPlAY_SWITCH);
        if (loopPlaySwitch == C.LOOPPLAY_ON) {
            rbLoopPlayOn.setChecked(true);
        } else {
            rbLoopPlayOff.setChecked(true);
        }

        final RadioButton rbVideoCacheOn = contentView.findViewById(R.id.rb_setting_videocache_on);
        final RadioButton rbVideoCacheOff = contentView.findViewById(R.id.rb_setting_videocache_off);
        boolean videoCacheSwitch = SharedPrefsStore.getBooleanData(context, C.KEY_VIDEOCACHE_SWITCH);
        if (videoCacheSwitch == C.VIDEOCACHE_ON) {
            rbVideoCacheOn.setChecked(true);
        } else {
            rbVideoCacheOff.setChecked(true);
        }
        
        final RadioButton rbTexture = contentView.findViewById(R.id.rb_setting_renderview_texture);
        final RadioButton rbSurface = contentView.findViewById(R.id.rb_setting_renderview_surface);

        final LinearLayout llSmartVision = contentView.findViewById(R.id.ll_smartvision);
        final RadioButton rbDisbaleSV = contentView.findViewById(R.id.rb_setting_smartvision_disable);
        final RadioButton rbHdr = contentView.findViewById(R.id.rb_setting_smartvision_hdr);

        int renderViewType = SharedPrefsStore.getIntData(context,
                C.KEY_RENDERVIEW_TYPE, C.RENDERVIEW_TEXTURE);
        if (renderViewType == C.RENDERVIEW_TEXTURE) {
            rbTexture.setChecked(true);
        } else {
            rbSurface.setChecked(true);
        }

        rbHdr.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    Toast.makeText(context, "当前设备支持HDR显示: " + HdrKitWrapper.isHdr10DisplaySupported(context)
                            + ", HLG显示: " + HdrKitWrapper.isHlgDisplaySupported(context), Toast.LENGTH_SHORT).show();
                }
            }
        });

        int smartVisionType = CommonUtils.getSmartVisionType(context);
        if (smartVisionType == C.SMARTVISION_DISABLE) {
            rbDisbaleSV.setChecked(true);
        } else {
            rbHdr.setChecked(true);
        }

        try {
            if (FeatureManager.getInstance().checkHdrFeatureValid(context, BaseApp.APP_ID)) {
                llSmartVision.setVisibility(View.VISIBLE);
            }
        } catch (Exception e) {
            llSmartVision.setVisibility(View.GONE);
        }

        contentView.findViewById(R.id.tv_pop_close).setOnClickListener(new OnClickListener() {

            @Override
            public void onClick(View arg0) {
                dismiss();
            }
        });
        contentView.findViewById(R.id.btn_confirm).setOnClickListener(new OnClickListener() {

            @Override
            public void onClick(View arg0) {
                SharedPrefsStore.saveData(contentView.getContext(), C.KEY_ORIENT_TYPE,
                        rbVerti.isChecked() ? C.ORIENT_PORTRAIT : C.ORIENT_HORIZONTAL);

                SharedPrefsStore.saveData(contentView.getContext(), C.KEY_CONTROLBAR_TYPE,
                        rbSimple.isChecked() ? C.BARTYPE_SIMPLE : C.BARTYPE_ADVANCE);

                SharedPrefsStore.saveData(contentView.getContext(), C.KEY_AUTORESUME_SWITCH,
                        rbAutoResumeOn.isChecked() ? C.AUTORESUME_ON : C.AUTORESUME_OFF);

                SharedPrefsStore.saveData(contentView.getContext(), C.KEY_LOOPPlAY_SWITCH,
                        rbLoopPlayOn.isChecked() ? C.LOOPPLAY_ON : C.LOOPPLAY_OFF);

                SharedPrefsStore.saveData(contentView.getContext(), C.KEY_VIDEOCACHE_SWITCH,
                        rbVideoCacheOn.isChecked() ? C.VIDEOCACHE_ON : C.VIDEOCACHE_OFF);

                
                SharedPrefsStore.saveData(contentView.getContext(), C.KEY_RENDERVIEW_TYPE,
                        rbTexture.isChecked() ? C.RENDERVIEW_TEXTURE : rbSurface.isChecked() ?
                        C.RENDERVIEW_SURFACE : C.RENDERVIEW_GLSURFACE);
                SharedPrefsStore.saveData(contentView.getContext(), C.KEY_SMARTVISION_TYPE,
                        rbDisbaleSV.isChecked() ? C.SMARTVISION_DISABLE : C.SMARTVISION_HDR);
                dismiss();
            }
        });
    }

    public static SettingsPopWindow createSettingsPopWindow(final SampleListActivity context) {
        LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View ctView = inflater.inflate(R.layout.pop_window_settings, null);
        SettingsPopWindow rtPopWindow = new SettingsPopWindow(context, ctView);
        return rtPopWindow;
    }

    @Override
    public void dismiss() {
        super.dismiss();
        WindowManager.LayoutParams lp = activity.getWindow().getAttributes();
        lp.alpha = 1f;
        activity.getWindow().setAttributes(lp);
    }

    /**
     * 显示popupWindow
     *
     */
    public void showPopupWindow() {
        if (!this.isShowing()) {
            // 以下拉方式显示popupwindow
            this.showAtLocation(activity.findViewById(R.id.rl_main), Gravity.CENTER, 0, 0);
            WindowManager.LayoutParams lp = activity.getWindow().getAttributes();
            lp.alpha = 0.3f;
            activity.getWindow().setAttributes(lp);
        } else {
            this.dismiss();
        }
    }
}
