package com.baidu.cloud.videoplayer.demo.rtc;

import android.app.Activity;
import android.graphics.Color;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class LogTextViewHelper {
    private static final String TAG = "LogTextViewHelper";
    private final TextView mStateInfoPanel2;
    private ScrollView mLogScrollView;
    private Activity mContext;
    protected TextView mStreamingInfoPanel;
    protected TextView mDebugLogPanel;
    protected TextView mStateInfoPanel;
    protected StringBuffer mLogStrBuffer;
    protected LinearLayout mDebugPanel;
    private boolean mStarted;

    public LogTextViewHelper(Activity context, LinearLayout debugPanel) {

        mContext = context;
        mLogStrBuffer = new StringBuffer();
        mDebugPanel = debugPanel;
        mStreamingInfoPanel = new TextView(mContext);
        LinearLayout.LayoutParams streamingInfoLp =
                new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        this.mStreamingInfoPanel.setLayoutParams(streamingInfoLp);
        this.mStreamingInfoPanel.setTextColor(Color.RED);

        mStateInfoPanel = new TextView(mContext);
        LinearLayout.LayoutParams stateInfoLp =
                new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        this.mStateInfoPanel.setPadding(0, 6, 0, 0);
        this.mStateInfoPanel.setLayoutParams(stateInfoLp);
        this.mStateInfoPanel.setTextColor(Color.RED);

        mStateInfoPanel2 = new TextView(mContext);
        LinearLayout.LayoutParams stateInfoLp2 =
                new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        this.mStateInfoPanel2.setPadding(0, 6, 0, 0);
        this.mStateInfoPanel2.setLayoutParams(stateInfoLp2);
        this.mStateInfoPanel2.setMovementMethod(ScrollingMovementMethod.getInstance());
        this.mStateInfoPanel2.setTextColor(Color.RED);
        this.mStateInfoPanel2.setVerticalScrollBarEnabled(true);
        this.mStateInfoPanel2.setScrollBarStyle(View.SCROLLBARS_OUTSIDE_OVERLAY);
        this.mStateInfoPanel2.setVerticalFadingEdgeEnabled(true);


        mDebugLogPanel = new TextView(mContext);
        FrameLayout.LayoutParams debugLogLp =
                new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                        ViewGroup.LayoutParams.MATCH_PARENT);
        this.mDebugLogPanel.setLayoutParams(debugLogLp);
        this.mDebugLogPanel.setTextColor(Color.RED);

        mLogScrollView = new ScrollView(mContext);
        LinearLayout.LayoutParams logScrollLp =
                new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                        ViewGroup.LayoutParams.MATCH_PARENT);
        this.mLogScrollView.setPadding(0, 6, 0, 0);
        this.mLogScrollView.setLayoutParams(logScrollLp);
        this.mLogScrollView.setVerticalScrollBarEnabled(true);
        this.mLogScrollView.setScrollbarFadingEnabled(true);
        mLogScrollView.addView(mDebugLogPanel);

        mDebugPanel.addView(mStreamingInfoPanel);
        mDebugPanel.addView(mStateInfoPanel);
        mDebugPanel.addView(mStateInfoPanel2);
        mDebugPanel.addView(mLogScrollView);
    }

    public void start() {
        if (mStarted) {
            return;
        }
        mStarted = true;
        mHandler.sendEmptyMessageDelayed(MSG_UPDATE_LOG, 1000);
    }

    public void stop() {
        if (!mStarted) {
            return;
        }
        mStarted = false;
        mHandler.removeMessages(MSG_UPDATE_LOG);
        mHandler.removeCallbacksAndMessages(null);
        mHandler = null;
        mContext = null;
    }

    protected static String formatedDurationMilli(long duration) {
        if (duration >= 1000) {
            return String.format(Locale.US, "%.2f sec", ((float) duration) / 1000);
        } else {
            return String.format(Locale.US, "%d msec", duration);
        }
    }

    protected static String formatedSpeed(long bytes, long elapsedMilli) {
        if (elapsedMilli <= 0) {
            return "0 B/s";
        }

        if (bytes <= 0) {
            return "0 B/s";
        }

        float bytesPerSec = ((float) bytes) * 1000.f / elapsedMilli;
        if (bytesPerSec >= 1000 * 1000) {
            return String.format(Locale.US, "%.2f MB/s", bytesPerSec / 1000 / 1000);
        } else if (bytesPerSec >= 1000) {
            return String.format(Locale.US, "%.1f KB/s", bytesPerSec / 1000);
        } else {
            return String.format(Locale.US, "%d B/s", (long) bytesPerSec);
        }
    }

    protected static String formatedSize(long bytes) {
        if (bytes >= 100 * 1000) {
            return String.format(Locale.US, "%.2f MB", ((float) bytes) / 1000 / 1000);
        } else if (bytes >= 100) {
            return String.format(Locale.US, "%.1f KB", ((float) bytes) / 1000);
        } else {
            return String.format(Locale.US, "%d B", bytes);
        }
    }

    protected static String formatLog(String logText) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH:mm:ss:SSS");
        String timeString = simpleDateFormat.format(new Date());
        String logMsg =  " [" + timeString + "] " + logText + "\n";
        return logMsg;
    }

    protected void updateLogInfo() {
        if (mStreamingInfoPanel != null) {
            mStreamingInfoPanel.setText(getLogInfoString());
        }
    }

    public void updateStateInfo(final String stateMsg) {
        if (mStateInfoPanel != null) {
            //            final String info =  formatLog(stateMsg);
            Log.d(TAG, stateMsg);
            if (mHandler != null) {
                mHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        mStateInfoPanel.setText(stateMsg);
                    }
                });
            }
        }
    }

    public void updateStateInfo2(final String stateMsg) {
        if (mStateInfoPanel2 != null) {
            //            final String info =  formatLog(stateMsg);
            Log.d(TAG, stateMsg);
            if (mHandler != null) {
                mHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        mStateInfoPanel2.setText(stateMsg);
                    }
                });
            }
        }
    }

    public void addLogText(String debugText) {
        if (mDebugLogPanel != null && !TextUtils.isEmpty(debugText)) {
            String logMsg =  formatLog(debugText);
            Log.d(TAG, logMsg);
            mLogStrBuffer.append(logMsg);
            if (mHandler != null) {
                mHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        mDebugLogPanel.setText(mLogStrBuffer);
                    }
                });
            }
        }
    }

    protected String getLogInfoString() {

        return getStateString() + getMediaInfoString() + getDebugString();
    }

    public void printDebugLog(String debugLog) {
        if (!TextUtils.isEmpty(debugLog)) {
            addLogText(debugLog);
        }
    }

    public void updateStateInfoLog(String infoMsg) {
        if (!TextUtils.isEmpty(infoMsg)) {
            updateStateInfo(infoMsg);
        }
    }

    public void updateStateInfoLog2(String infoMsg) {
        if (!TextUtils.isEmpty(infoMsg)) {
            updateStateInfo2(infoMsg);
        }
    }

    protected String getDebugString() {
        return "";
    }

    protected String getMediaInfoString() {
        return "";
    }

    protected String getStateString() {
        return "";
    }

    private static final int MSG_UPDATE_LOG = 1;

    private Handler mHandler = new Handler() {

        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case MSG_UPDATE_LOG: {
                    if (!mStarted) {
                        break;
                    }
                    updateLogInfo();
                    mHandler.removeMessages(MSG_UPDATE_LOG);
                    mHandler.sendEmptyMessageDelayed(MSG_UPDATE_LOG, 1000);
                }
            }
        }
    };
}
