package com.baidu.cloud.videoplayer.demo.rtc;

import java.util.ArrayList;

import com.baidu.cloud.videoplayer.demo.R;
import com.baidu.cloud.videoplayer.demo.rtc.fragment.RTCFragmentAdapter;
import com.baidu.cloud.videoplayer.demo.rtc.fragment.RTCVideoFragment;
import com.baidu.rtc.player.BRTCPlayerImpl;
import com.webrtc.Logging;

import com.baidu.rtc.RTCLoadManager;

import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.viewpager.widget.ViewPager;

public class RTCPlayerPagerActivity extends AppCompatActivity {
    private static final String TAG = "RTCPlayerFragmentActivity";
    /**
     * 修改该值即可模拟翻页场景
     */
    private static final int FAKE_FRAGMENTS_COUNT = 1;
    private ViewPager mVideoViewPager;
    private int mOffScreenLimit = 1;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Window win = getWindow();
        win.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        win.requestFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.activity_rtcplayer_pager);
        RTCLoadManager.getInstance(this).setDefaultCpuType("arm64-v8a");
        RTCLoadManager.getInstance(this).registerCallback(mLoadListener);
        RTCLoadManager.getInstance(this).loadLibraries(BRTCPlayerImpl.getDefaultSoDownloadUrl(), "arm64-v8a", null);

        mVideoViewPager = (ViewPager) findViewById(R.id.rtc_video_view_pager);
        initVideoViewPager();
    }

    private void initVideoViewPager() {
        ArrayList<Fragment> fragmentList = new ArrayList<>();
        for (int pos = 0; pos < FAKE_FRAGMENTS_COUNT; pos++) {
            RTCVideoFragment fragment = new RTCVideoFragment();
            fragment.setPosition(pos % 3);
            fragmentList.add(fragment);
        }
        RTCFragmentAdapter adapter = new RTCFragmentAdapter(getSupportFragmentManager(), fragmentList);
        mVideoViewPager.setAdapter(adapter);
        mVideoViewPager.setOffscreenPageLimit(mOffScreenLimit);

    }

    private RTCLoadManager.LoadListener mLoadListener = new RTCLoadManager.LoadListener() {

        @Override
        public void onLoadError(int errCode, String errMsg) {
            Logging.d(TAG, "external load library error " + errCode + errMsg);
        }

        @Override
        public void onLibsDownloadCompleted() {
            Logging.d(TAG, "libs download completed.");
        }

        @Override
        public void onLoadSuccess() {
            Logging.d(TAG, "external load library success.");
        }

        @Override
        public void onLoadProgress(float progress) {
            Logging.d(TAG, "external load library so progress " + progress);
        }
    };

    @Override
    protected void onDestroy() {
        super.onDestroy();
        RTCLoadManager.getInstance(this).unregisterCallback(mLoadListener);
    }

    public void onClickClose(View v) {
        finish();
    }
}