package com.baidu.cloud.videoplayer.demo.rtc.fragment;

import android.content.Context;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Bundle;

import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;

import android.os.Handler;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.baidu.cloud.videoplayer.demo.BaseApp;
import com.baidu.cloud.videoplayer.demo.C;
import com.baidu.cloud.videoplayer.demo.R;
import com.baidu.cloud.videoplayer.demo.helper.HttpDnsUtil;
import com.baidu.cloud.videoplayer.demo.rtc.LogTextViewHelper;
import com.baidu.rtc.RTCVideoView;
import com.baidu.rtc.player.BRTCPlayer;
import com.baidu.rtc.player.BRTCPlayerEvents;
import com.baidu.rtc.player.BRTCPlayerImpl;
import com.baidu.rtc.player.BRTCPlayerParameters;
import com.baidu.rtc.player.PlayTimeStatistician;
import com.baidu.rtc.player.RTCCommStatesReport;

import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.util.Arrays;


public class RTCVideoFragment extends Fragment implements BRTCPlayerEvents {
    private static final String TAG = "RTCVideoFragment";

    private boolean mIsViewCreated;
    private boolean mIsUIVisible;
    private RTCVideoView mVideoView;
    protected BRTCPlayerImpl mBRTCPlayer;
    private BRTCPlayerParameters mPlayerParameters;
    private RTCVideoView.ScalingType mScaleMode = RTCVideoView.ScalingType.SCALE_ASPECT_FIT;
    private FragmentActivity mContext;
    private String mStreamUrl;
    private String mSignalUrl;
    private LinearLayout mDebugPanelTv;
    private Handler mHandler;
    private LogTextViewHelper mDebugLogHelper;
    private boolean mIsRepeating;
    private boolean bUseLssUdp;
    private int mPos;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_rtcvideo, container, false);
        mContext = getActivity();
        initData();
        initView(view);
        initPlayer();
        return view;
    }

    private void initData() {
        mStreamUrl = mContext.getIntent().getExtras().getString(C.KEY_RTC_STREAMURL);
        if (mStreamUrl == null || mStreamUrl.isEmpty()) {
            mStreamUrl = "";
        }
        Log.d(TAG, "stream url: " + mStreamUrl);

        mSignalUrl = mContext.getIntent().getExtras().getString(C.KEY_RTC_SIGNALURL);
        if (mSignalUrl == null || mSignalUrl.isEmpty()) {
            mSignalUrl = "";
        }
        Log.d(TAG, "signal url: " + mSignalUrl);

        if (C.RTC_SIGNAL_TYPE_UDP.equals(mContext.getIntent().getExtras().getString(C.KEY_RTC_SIGNAL_TYPE))) {
            bUseLssUdp = true;
        }

        HttpDnsUtil.getIpFromDomain(Uri.parse(mStreamUrl).getHost(), new HttpDnsUtil.IDnsCallback() {
            @Override
            public void onSuccess(HttpDnsUtil.IpInfo ipInfo) {
                // TODO consider TTL
                if (ipInfo.ipAddrs.size() > 0) {
                    Log.d(TAG, "successfully got Ip(" + ipInfo.ipAddrs.get(0) + ")");
                    saveIpData(mContext, Uri.parse(mStreamUrl).getHost(), ipInfo.ipAddrs.get(0));
                }
            }

            @Override
            public void onFail() {

            }
        });
    }

    private void initView(View view) {
        printDebugLog("init view.");
        TextView streamInfoTv = view.findViewById(R.id.tv_stream_info);
        streamInfoTv.setText("StreamUrl:" + mStreamUrl + ", SignalUrl:" + mSignalUrl);
        mDebugPanelTv = view.findViewById(R.id.rtc_debug_tv);
        mDebugLogHelper = new LogTextViewHelper(mContext, mDebugPanelTv);
        mVideoView = view.findViewById(R.id.brtc_video_view);

        streamInfoTv.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mDebugPanelTv.setVisibility(mDebugPanelTv.getVisibility() == View.VISIBLE ?
                        View.INVISIBLE : View.VISIBLE);
            }
        });
        mHandler = new Handler(mContext.getMainLooper());
    }

    protected void initPlayer() {
        printDebugLog("init player.");
        mBRTCPlayer = new BRTCPlayerImpl(mContext, BaseApp.APP_ID);
        mPlayerParameters = new BRTCPlayerParameters();
        mPlayerParameters.enableSoLaterLoad(true);
        mPlayerParameters.setCpuType("arm64-v8a");
        mPlayerParameters.setEnableDebug(true);
        mPlayerParameters.setPullUrl(mSignalUrl);
        mPlayerParameters.setEnableEncrypt(false);
        String ip = getIpData(mContext, Uri.parse(mStreamUrl).getHost());
        if (!TextUtils.isEmpty(ip)) {
            mPlayerParameters.setMediaServerIp(ip);
        }
        if (bUseLssUdp) {
            mPlayerParameters.setEnableSignalOverUdp(true);
        }

        mBRTCPlayer.initPlayer(mPlayerParameters, this);
        mBRTCPlayer.setStreamUri(mStreamUrl);
        mBRTCPlayer.setSurfaceView(mVideoView);

        /**
         * 预设RTC媒体服务器地址,在调用{@link #startPlay()}接口播放前调用,用于优化启播时间, 非必需调用.
         * 获取 mediaserverip 方式可以选择使用 httpdns 解析,
         * 请求方式为：GET http://<httpdns_interface>/?dns=rtc-ms-edge.exp.bcelive.com,
         * 其中 rtc-ms-edge.exp.bcelive.com 是超低延时直播媒体服务器固定调度域名。
         */
        //        mBRTCPlayer.setMediaServerIp("");
        //        mBRTCPlayer.setMediaServerIp("119.167.192.131");
        mBRTCPlayer.setScalingType(mScaleMode);
        mBRTCPlayer.prepareAsync();
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        printDebugLog("view created.");
        mIsViewCreated = true;
        tryPlay();
    }

    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        printDebugLog("User visible hint " + isVisibleToUser);
        if (isVisibleToUser) {
            mIsUIVisible = true;
            tryPlay();
        } else {
            mIsUIVisible = false;
            if (mBRTCPlayer != null) {
                printDebugLog("stop play.");
                mBRTCPlayer.stopPlay();
            }
        }
    }

    private void tryPlay() {
        if (isViewVisible()) {
            Log.d(TAG, "view visible try play");
            printDebugLog("view visible try play");
            if (mBRTCPlayer.getPlayerState() == BRTCPlayer.PlayerState.STATE_PREPARED) {
                printDebugLog("start real play.");
                mBRTCPlayer.startPlay();
            } else if (mBRTCPlayer.getPlayerState() == BRTCPlayer.PlayerState.STATE_STOPPED
                    || mBRTCPlayer.getPlayerState() == BRTCPlayer.PlayerState.STATE_INITIALIZED) {
                mBRTCPlayer.prepareAsync();
            }
        }
    }

    private boolean isViewVisible() {
        boolean isViewVisible = mIsViewCreated && mIsUIVisible;
        printDebugLog("isViewVisible state:" + isViewVisible);
        return isViewVisible;
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        printDebugLog("destroy view release player");
        mIsViewCreated = false;
        mIsUIVisible = false;
        mDebugLogHelper.stop();
        mDebugLogHelper = null;
        mBRTCPlayer.stopPlay();
        mBRTCPlayer.releasePlayer();
        mVideoView = null;
        mDebugPanelTv = null;
        mHandler.removeCallbacksAndMessages(null);
        mHandler = null;
    }

    @Override
    public void onPrepared() {
        printDebugLog("play prepared.");
        if (isViewVisible()) {
            if (mBRTCPlayer != null) {
                printDebugLog("start play onprepared");
                mBRTCPlayer.startPlay();
            }
            if (mIsRepeating) {
                mIsRepeating = false;
            }
        }
    }

    @Override
    public void onFirstFrameRendered() {
        printDebugLog("on first frame rendered.");
    }

    @Override
    public void onResolutionChanged(int w, int h) {
        printDebugLog("on resolution changed  : " + w + " x " + h);
    }

    @Override
    public void onError(final int errCode, final String msg) {
        printDebugLog("onError  : errCode" + errCode + " errMsg: " + msg);
        mContext.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(mContext, errCode + " : " + msg, Toast.LENGTH_LONG).show();
            }
        });

        // 媒体通道出错 及流中断, 重新播放
        if (errCode == BRTCPlayerEvents.BRTC_PLAYER_ERROR_ICE_CHANNEL
                || errCode == BRTCPlayerEvents.BRTC_PLAYER_ERROR_STREAMING_INTERRUPT) {
            repeat();
        }
    }

    // 延迟2000ms 重新播放
    private void repeat() {
        mHandler.postDelayed(new Runnable() {
            @Override
            public void run() {
                if (mBRTCPlayer != null
                        && mBRTCPlayer.getPlayerState() == BRTCPlayer.PlayerState.STATE_ERROR) {
                    printDebugLog("start repeat ...");
                    mBRTCPlayer.stopPlay();
                    mBRTCPlayer.prepareAsync();
                    mIsRepeating = true;
                }
            }
        }, 2000);
    }

    @Override
    public void onInfoUpdated(int event, Object msg) {
        if (msg == null) {
            return;
        }
        if (event == BRTC_PLAYER_EVENT_STATS_UPDATED) {
            RTCCommStatesReport statesReport = (RTCCommStatesReport) msg;
            Log.d(TAG, "stats updated : "
                    + statesReport.getVideoOutputFps() + " / "
                    + statesReport.getVideoRecvBitrate() + " / "
                    + statesReport.getVideoOutputFps() + " / "
                    + statesReport.getAudioRecvBitrate() + " / "
                    + statesReport.getConnRecvBitrate() + " / "
                    + statesReport.getVideoRecvPacketLost() + " / "
                    + statesReport.getVideoPacketLostRatioPerMil() + " / "
                    + statesReport.getRoomId() + " / "
                    + statesReport.getUserId() + " / "
                    + statesReport.getSessionId() + " / "
                    + statesReport.getSignallingType() + " / "
                    + statesReport.getAudioJitterBufferMs() + " / "
                    + statesReport.getRemoteAddr());
            if (mDebugLogHelper != null) {
                mDebugLogHelper.updateStateInfoLog(statesReport.toString());
            }
        } else if (event == BRTC_PLAYER_EVENT_PLAY_TIME_STATISTIC) {
            PlayTimeStatistician timeStatistician = (PlayTimeStatistician) msg;
            printDebugLog("first frame time consume : " + timeStatistician.getRealFirstFrameTime());
            printDebugLog("time statistic updated : " + timeStatistician.toString());
            printDebugLog("time statistic start time : " + timeStatistician.getStartTime());
            printDebugLog("time statistic end time : " + timeStatistician.getEndTime());
            printDebugLog("time statistic signalling type : "
                    + (timeStatistician.getSignallingType() == 1 ? "udp" : "http"));
            printDebugLog("time statistic session id : " + timeStatistician.getSessionId());
            printDebugLog("time statistic remote ip : " + timeStatistician.getRemoteIp());
        } else if (event == BRTC_PLAYER_EVENT_BUFFERING_END) {
            long bufferingDuraton = (long) msg;
            printDebugLog("buffering end, buffering duration : " + bufferingDuraton + "ms");
        } else {
            if (event != BRTC_PLAYER_EVENT_LOCAL_SDP_SET) {
                printDebugLog("on info updated : " + event + " msg:" + msg);
            }
        }
        // Log.d(TAG, "onInfoUpdated  : " + event + " msg:" + msg);
    }

    @Override
    public void onRemoteStreamStats(boolean hasVideo, boolean hasAudio, BigInteger handleID) {
        printDebugLog("onRemoteStreamStats  video:" + hasVideo + " audio:" + hasAudio);
    }

    @Override
    public void onRemoteData(ByteBuffer byteBuffer) {
        Log.d(TAG, "onRemoteData  received!");
    }

    @Override
    public void onPlayerStateChanged(BRTCPlayer.PlayerState state) {
        printDebugLog("Player state changed  to " + state.toString());
    }

    /**
     * 接收到SEI 消息
     *
     * @param data SEI 数据
     *             格式:[nal_type(1)|sei_type(1)|sei_size(n+1)|sei_payload(n*255+m)|trailing_bits
     *             (1)]
     *             sei_size: 需判断第3个字节起连续0xFF的个数n,加第1个不为0xFF的字节(如: 0x21),
     *             该字段占字节数为 n + 1;  其值为:0xFF*n+0x21;
     *             sei_payload: sei_size字段的值，
     *             如:[ 0x06 0x05 0x18 0x54 0x80 0x83 0x97 0xF0 0x23 0x47 0x4B 0xB7 0xF7 0x4F 0x32
     *             0xB5 0x4E 0x06 0xAC 0x27 0x11 0xFE 0x69 0x79 0x01 0x00 0x00 0x80 ]
     *
     * @apiNote 回调在解码线程, 不应在该回调里实现复杂业务
     */
//    @RequiresApi(api = Build.VERSION_CODES.O)
    @Override
    public void onSEIRecv(ByteBuffer data) {
        if (data == null) {
            return;
        }
        byte[] seiData = Arrays.copyOfRange(data.array(),
                data.arrayOffset(),
                data.capacity());
        printBytes(TAG, seiData, seiData.length);

        final int seiLen = seiData.length;
        int index = 2;
        int realPayloadSize = 0;
        do {
            realPayloadSize += seiData[index] & 0xFF;
        } while (seiData[index++] == (byte) 0xFF);

        final String seiMsg = new String(seiData, 2 + (index - 2) + 16, realPayloadSize - 16);
        final int finalRealPayloadSize = realPayloadSize;
        mContext.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                String seiInfo =
                        "Recv SEI " + seiLen + "/" + finalRealPayloadSize
                                + " Msg: \n [ " + seiMsg + " ] ";
                Log.d(TAG, seiInfo);
                // Toast.makeText(RTCPlayerActivity.this, seiInfo, Toast.LENGTH_LONG).show();
            }
        });
    }

    private void printBytes(String tag, byte[] bb, int size) {
        StringBuilder sb = new StringBuilder();
        int i = 0;
        int bytesInLine = 16;
        int max = bb.length;
        for (i = 0; i < size && i < max; i++) {
            sb.append(String.format("0x%02X ", bb[i] & 0xFF));
            if (((i + 1) % bytesInLine) == 0) {
                Log.i(tag, String.format("%03d-%03d: %s", i / bytesInLine * bytesInLine, i,
                        sb.toString()));
                sb = new StringBuilder();
            }
        }
        if (sb.length() > 0) {
            Log.i(tag, String.format("%03d-%03d: %s", size / bytesInLine * bytesInLine, i - 1,
                    sb.toString()));
        }
    }

    public void setPosition(int position) {
        mPos = position;
    }

    public void printDebugLog(String debugLog) {
        if (mDebugLogHelper != null && !TextUtils.isEmpty(debugLog)) {
            String streamName = "";
            if (mStreamUrl.startsWith("webrtc://")) {
                streamName = mStreamUrl.substring(mStreamUrl.lastIndexOf("/") + 1);
            }
            mDebugLogHelper.printDebugLog( "[" + streamName + "]" + debugLog);
        }
    }

    public static void saveIpData(Context context, String domain, String ip) {
        SharedPreferences spList = context.getSharedPreferences(C.KEY_HTTP_DNS, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = spList.edit();
        editor.putString(domain, ip);
        editor.apply();
    }

    public static String getIpData(Context context, String domain) {
        SharedPreferences spList = context.getSharedPreferences(C.KEY_HTTP_DNS, Context.MODE_PRIVATE);
        return spList.getString(domain, "");
    }
}
