package com.baidu.cloud.videoplayer.demo.virtuallive;

import java.io.IOException;

import com.baidu.cloud.media.player.BDCloudMediaPlayer;
import com.baidu.cloud.media.player.IMediaPlayer;
import com.baidu.cloud.videoplayer.demo.BaseApp;
import com.baidu.cloud.videoplayer.demo.C;
import com.baidu.cloud.videoplayer.demo.R;
import com.baidu.cloud.videoplayer.demo.info.SharedPrefsStore;
import com.baidu.cloud.videoplayer.widget.IRenderView;
import com.baidu.cloud.videoplayer.widget.VirtualLiveRenderView;

import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.TextView;
import androidx.appcompat.app.AppCompatActivity;
import android.os.Bundle;

public class VirtualLiveActivity extends AppCompatActivity implements IMediaPlayer.OnPreparedListener,
        IMediaPlayer.OnVideoSizeChangedListener{
    private static final String TAG = "VirtualLiveActivity";
    private static final String FG_GREEN_SCREEN_VIDEO_URL =
            "https://bdcloud-player.cdn.bcebos.com/testvideo/mp4/greenscreen/mixkit-girl.mp4";
    private static final String BG_VIDEO_URL =
            "https://bdcloud-player.cdn.bcebos.com/testvideo/hls/265/1080p/cctv/test_hls_h265_1080p_output.m3u8";
    private RelativeLayout mViewHolder;
    private EditText mBgUrlEditText;
    private EditText mFgUrlEditText;
    private Button mPlayBtn;
    private Button mStopBtn;
    private VirtualLiveRenderView mVirtualLiveRenderView;
    private BDCloudMediaPlayer mMediaPlayer;
    private BDCloudMediaPlayer mBgMediaPlayer;
    private boolean bPreparing;
    private boolean bPlaying;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_virtual_live);

        mFgUrlEditText = findViewById(R.id.fg_video_url);
        mFgUrlEditText.setText(SharedPrefsStore.getData(this, C.KEY_VIRTUAL_LIVE_FG_URL, FG_GREEN_SCREEN_VIDEO_URL));
        mFgUrlEditText.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int id, KeyEvent keyEvent) {
                if (id == EditorInfo.IME_ACTION_DONE || id == EditorInfo.IME_NULL) {

                    if (mFgUrlEditText.getText().toString().isEmpty()) {
                        return true;
                    }
                    SharedPrefsStore.saveData(getApplicationContext(), C.KEY_VIRTUAL_LIVE_FG_URL,
                            mFgUrlEditText.getText().toString());
                    return true;
                }
                return false;
            }
        });

        mBgUrlEditText = findViewById(R.id.bg_video_url);
        mBgUrlEditText.setText(SharedPrefsStore.getData(this, C.KEY_VIRTUAL_LIVE_BG_URL,
                BG_VIDEO_URL));
        mBgUrlEditText.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int id, KeyEvent keyEvent) {
                if (id == EditorInfo.IME_ACTION_DONE || id == EditorInfo.IME_NULL) {

                    if (mBgUrlEditText.getText().toString().isEmpty()) {
                        return true;
                    }
                    SharedPrefsStore.saveData(getApplicationContext(), C.KEY_VIRTUAL_LIVE_BG_URL,
                            mBgUrlEditText.getText().toString());
                    return true;
                }
                return false;
            }
        });

        mPlayBtn = (Button) findViewById(R.id.virtual_live_play_button);
        mPlayBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (bPlaying) {
                    return;
                }
                bPlaying = true;

                if (mFgUrlEditText.getText().toString().isEmpty() || mBgUrlEditText.getText().toString().isEmpty()) {
                    return;
                }
                SharedPrefsStore.saveData(getApplicationContext(), C.KEY_VIRTUAL_LIVE_FG_URL,
                        mFgUrlEditText.getText().toString());
                SharedPrefsStore.saveData(getApplicationContext(), C.KEY_VIRTUAL_LIVE_BG_URL,
                        mBgUrlEditText.getText().toString());

                mViewHolder = findViewById(R.id.virtual_live_view_holder);

                mVirtualLiveRenderView = new VirtualLiveRenderView(getApplicationContext());
                RelativeLayout.LayoutParams rllp = new RelativeLayout.LayoutParams(
                        ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
                rllp.addRule(RelativeLayout.CENTER_HORIZONTAL);
                mViewHolder.addView(mVirtualLiveRenderView, 0, rllp);

                mVirtualLiveRenderView.addRenderCallback(mSHCallback);
            }
        });

        mStopBtn = (Button) findViewById(R.id.virtual_live_stop_button);
        mStopBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (!bPlaying) {
                    return;
                }
                bPlaying = false;

                if (mVirtualLiveRenderView != null) {
                    mVirtualLiveRenderView.release();
                    mVirtualLiveRenderView.destroy();
                    mVirtualLiveRenderView.removeRenderCallback(mSHCallback);
                }
                mViewHolder.removeView(mVirtualLiveRenderView);
            }
        });

        BDCloudMediaPlayer.setAppId(BaseApp.APP_ID);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mMediaPlayer != null) {
            mMediaPlayer.stop();
            mMediaPlayer.release();
            mMediaPlayer = null;
        }
        if (mBgMediaPlayer != null) {
            mBgMediaPlayer.stop();
            mBgMediaPlayer.release();
            mBgMediaPlayer = null;
        }
        if (mVirtualLiveRenderView != null) {
            mVirtualLiveRenderView.release();
            mVirtualLiveRenderView.destroy();
            mVirtualLiveRenderView.removeRenderCallback(mSHCallback);
        }
    }

    IRenderView.IRenderCallback mSHCallback = new IRenderView.IRenderCallback() {
        @Override
        public void onSurfaceChanged(IRenderView.ISurfaceHolder holder, int format, int w, int h) {
            Log.d(TAG, "mSHCallback onSurfaceChanged " + w + ", " + h);
            if (holder.getRenderView() != mVirtualLiveRenderView) {
                Log.e(TAG, "onSurfaceChanged: unmatched render callback\n");
                return;
            }

            boolean hasValidSize = !mVirtualLiveRenderView.shouldWaitForResize() || (w > 0 && h > 0);

            if (hasValidSize && !bPreparing) {
                if (mMediaPlayer != null) {
                    mMediaPlayer.prepareAsync();
                }
                if (mBgMediaPlayer != null) {
                    mBgMediaPlayer.prepareAsync();
                }
                bPreparing = true;
            }
        }

        @Override
        public void onSurfaceCreated(IRenderView.ISurfaceHolder holder, int width, int height) {
            Log.d(TAG, "mSHCallback onSurfaceCreated " + width + ", " + height);
            if (holder.getRenderView() != mVirtualLiveRenderView) {
                Log.e(TAG, "onSurfaceCreated: unmatched render callback\n");
                return;
            }

            mMediaPlayer = new BDCloudMediaPlayer(VirtualLiveActivity.this);
            mMediaPlayer.setOnPreparedListener(VirtualLiveActivity.this);
            mMediaPlayer.setOnVideoSizeChangedListener(VirtualLiveActivity.this);
            mMediaPlayer.setLooping(true);

            mBgMediaPlayer = new BDCloudMediaPlayer(VirtualLiveActivity.this);
            mBgMediaPlayer.setOnPreparedListener(VirtualLiveActivity.this);
            mBgMediaPlayer.setOnVideoSizeChangedListener(VirtualLiveActivity.this);
            mBgMediaPlayer.setLooping(true);

            if (mMediaPlayer != null) {
                try {
                    mMediaPlayer.setDataSource(mFgUrlEditText.getText().toString());
                    mMediaPlayer.setSurface(mVirtualLiveRenderView.getSurfaceHolder().openSurface());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            if (mBgMediaPlayer != null) {
                try {
                    mBgMediaPlayer.setDataSource(mBgUrlEditText.getText().toString());
                    mBgMediaPlayer.setSurface(mVirtualLiveRenderView.openBgSurface());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        @Override
        public void onSurfaceDestroyed(IRenderView.ISurfaceHolder holder) {
            if (holder.getRenderView() != mVirtualLiveRenderView) {
                Log.e(TAG, "onSurfaceDestroyed: unmatched render callback\n");
                return;
            }
            Log.e(TAG, "onSurfaceDestroyed\n");

            if (mMediaPlayer != null) {
                mMediaPlayer.stop();
                mMediaPlayer.release();
                mMediaPlayer = null;
            }
            if (mBgMediaPlayer != null) {
                mBgMediaPlayer.stop();
                mBgMediaPlayer.release();
                mBgMediaPlayer = null;
            }
            bPreparing = false;
        }
    };

    @Override
    public void onPrepared(IMediaPlayer mp) {
        mp.start();
    }

    @Override
    public void onVideoSizeChanged(IMediaPlayer mp, int width, int height, int sarNum,
                                   int sarDen) {
        if (mp.equals(mMediaPlayer)) {
            mVirtualLiveRenderView.setAspectRatio(IRenderView.AR_ASPECT_FIT_PARENT);
            mVirtualLiveRenderView.setVideoSize(width, height);
            mVirtualLiveRenderView.setVideoSampleAspectRatio(sarNum, sarDen);
        }
    }
}