package com.baidu.cloud.videoplayer.widget;

import android.app.Activity;
import android.content.Context;
import android.media.AudioManager;
import android.os.Handler;
import android.os.Message;
import android.util.AttributeSet;
import android.util.Log;
import android.view.GestureDetector;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.baidu.cloud.videoplayer.demo.R;
import com.baidu.cloud.videoplayer.demo.bar.AdvancedMediaController;
import com.baidu.cloud.videoplayer.demo.helper.CommonUtils;

public class DefaultGestureView extends FrameLayout implements Handler.Callback, View.OnTouchListener {
    private static final String TAG = "DefaultGestureView";
    private LinearLayout container;
    private ImageView ivIcon;
    private TextView tvPercent;
    private ProgressBar pbPercent;
    private Handler handler;

    private GestureDetector mGestureDetector;
    private AudioManager mAudioManager;

    private int mCurVolume = -1;
    private float mCurBrightness;
    private int mSeekPosition = -1;
    private float mCurSpeed;

    // 处理单指逻辑
    private boolean bFirstTouch;
    private boolean bChangePosition;
    private boolean bChangeBrightness;
    private boolean bChangeVolume;

    private static final float SCALE_MAX = 2.0f;
    private static final float SCALE_MIN = 1.0f;
    // 处理双指逻辑
    private double mOldDist = 0;
    private double mMoveDist = 0;
    private float mDownX1 = 0;
    private float mDownX2 = 0;
    private float mDownY1 = 0;
    private float mDownY2 = 0;

    private boolean bHandlingOneTouch;
    private boolean bHandlingLongPress;
    private boolean bHandlingTwoTouch;

    private Activity mContext;

    private static final int SEEK_MAX = 100;
    private static final int ACTION_FAST_FORWARD = 1;
    private static final int ACTION_FAST_REWARD = 2;
    private static final int ACTION_FADE_DIALOG = 3;
    private static final int DIALOG_SHOW_TIME = 1 * 1000;

    private AdvancedMediaController mMediaController;

    public interface OnGestureViewClickListener {
        void onGestureViewClick();
    }

    private OnGestureViewClickListener mOnClickListener;

    public DefaultGestureView(Context context) {
        super(context);
        initView(context);
    }

    public DefaultGestureView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initView(context);
    }

    public DefaultGestureView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initView(context);
    }

    private void initView(Context context) {
        LayoutInflater.from(context).inflate(R.layout.view_gesture, this);
        container = findViewById(R.id.center_container);
        ivIcon = findViewById(R.id.iv_icon);
        tvPercent = findViewById(R.id.tv_percent);
        pbPercent = findViewById(R.id.pro_percent);
        handler = new Handler(this);
        pbPercent.setMax(SEEK_MAX);
        container.setVisibility(View.INVISIBLE);

        mContext = (Activity) context;
        mAudioManager = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);
        mGestureDetector = new GestureDetector(mContext, new VideoPlayGestureListener());

        setOnTouchListener(this);
    }

    public void setOnGestureClickListener(OnGestureViewClickListener listener) {
        mOnClickListener = listener;
    }

    public void setMediaController(AdvancedMediaController controller) {
        mMediaController = controller;
    }

    @Override
    public boolean handleMessage(Message msg) {
        int duration;
        int position;
        switch (msg.what) {
            case ACTION_FADE_DIALOG:
                container.setVisibility(View.INVISIBLE);
                break;
            case ACTION_FAST_FORWARD:
                handler.removeMessages(ACTION_FADE_DIALOG);

                mMediaController.setSpeed(3.0f);

                duration = mMediaController.getDuration();
                if (duration > 0) {
                    position = mMediaController.getCurrentPosition();
                    pbPercent.setProgress(100 * position / duration);
                    tvPercent.setText(String.format("%s/%s", CommonUtils.stringForTime(position),
                            CommonUtils.stringForTime(duration)));
                }
                handler.sendEmptyMessageDelayed(ACTION_FAST_FORWARD, 1000);
                break;
            case ACTION_FAST_REWARD:
                handler.removeMessages(ACTION_FADE_DIALOG);

                int curpos = mMediaController.getCurrentPosition();
                curpos = Math.max(0, curpos - 5000);
                mMediaController.seek(curpos);

                duration = mMediaController.getDuration();
                if (duration > 0) {
                    position = mMediaController.getCurrentPosition();
                    pbPercent.setProgress(100 * position / duration);
                    tvPercent.setText(String.format("%s/%s", CommonUtils.stringForTime(position),
                            CommonUtils.stringForTime(duration)));
                }
                handler.sendEmptyMessageDelayed(ACTION_FAST_REWARD, 1000);
                break;
            default:
                break;
        }
        return false;
    }


    class VideoPlayGestureListener extends GestureDetector.SimpleOnGestureListener {
        private float fx;
        private float fy;

        @Override
        public boolean onDown(MotionEvent e) {
            if (CommonUtils.isScreenEdge(getContext(), e)) {
                return true;
            }

            fx = e.getX();
            fy = e.getY();

            mCurVolume = mAudioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
            mCurBrightness = mContext.getWindow().getAttributes().screenBrightness;

            bFirstTouch = true;
            bChangePosition = false;
            bChangeBrightness = false;
            bChangeVolume = false;

            return true;
        }

        @Override
        public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX, float distanceY) {
            if (CommonUtils.isScreenEdge(getContext(), e1) || bHandlingLongPress) {
                return true;
            }

            bHandlingOneTouch = true;
            float deltaX = e1.getX() - e2.getX();
            float deltaY = e1.getY() - e2.getY();
            container.setVisibility(View.VISIBLE);
//            Log.v(TAG, "fx: = " + fx + ", deltaX= " + deltaX + ", deltaY= " + deltaY);
            if (bFirstTouch) {
                bChangePosition = Math.abs(distanceX) >= Math.abs(distanceY);
                if (!bChangePosition) {
                    int halfScreen = CommonUtils.getScreenWidth(mContext, false) / 2;
                    if (e2.getX() > halfScreen) {
                        bChangeVolume = true;
                    } else {
                        bChangeBrightness = true;
                    }
                }
                if (bChangePosition) {
                    bChangePosition = true;
                }
                bFirstTouch = false;
            }
            if (bChangePosition) {
                slideToChangePosition(deltaX);
            } else if (bChangeBrightness) {
                slideToChangeBrightness(deltaY);
            } else if (bChangeVolume) {
                slideToChangeVolume(deltaY);
            }
            return true;
        }

        @Override
        public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
            return false;
        }

        @Override
        public boolean onSingleTapConfirmed(MotionEvent e) {
            if (mOnClickListener != null) {
                mOnClickListener.onGestureViewClick();
            }
            return false;
        }

        @Override
        public boolean onSingleTapUp(MotionEvent e) {
            return false;
        }

        @Override
        public boolean onDoubleTap(MotionEvent e) {
            mMediaController.togglePlayPause();
            return true;
        }

        @Override
        public boolean onDoubleTapEvent(MotionEvent e) {
            return false;
        }

        public void onLongPress(MotionEvent e) {
            if (CommonUtils.isScreenEdge(getContext(), e) || bHandlingTwoTouch) {
                return;
            }
            bHandlingOneTouch = true;
            bHandlingLongPress = true;
            longPressToFFAndFR(e);
        }
    }

    private void slideToChangeVolume(float deltaY) {
        int streamMaxVolume = mAudioManager.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
        int height = getMeasuredHeight();
        float deltaV = deltaY * 2 / height * streamMaxVolume;
        float index = mCurVolume + deltaV;
        if (index > streamMaxVolume) {
            index = streamMaxVolume;
        }
        if (index < 0) {
            index = 0;
        }
        int percent = (int) (index / streamMaxVolume * 100);
        mAudioManager.setStreamVolume(AudioManager.STREAM_MUSIC, (int) index, 0);
        pbPercent.setProgress(percent);
        tvPercent.setText(percent + "%");
        ivIcon.setImageDrawable(getResources().getDrawable(R.drawable.btn_sound_normal));
        ivIcon.setVisibility(VISIBLE);
    }

    private void slideToChangeBrightness(float deltaY) {
        Window window = mContext.getWindow();
        WindowManager.LayoutParams attributes = window.getAttributes();
        int height = getMeasuredHeight();
        if (mCurBrightness == -1.0f) {
            mCurBrightness = 0.5f;
        }
        float brightness = deltaY / height * 1.0f + mCurBrightness;
        if (brightness < 0) {
            brightness = 0f;
        }
        if (brightness > 1.0f) {
            brightness = 1.0f;
        }
        int percent = (int) (brightness * 100);
        attributes.screenBrightness = brightness;
        window.setAttributes(attributes);
        pbPercent.setProgress(percent);
        tvPercent.setText(percent + "%");
        ivIcon.setImageDrawable(getResources().getDrawable(R.drawable.btn_bright_normal));
        ivIcon.setVisibility(VISIBLE);
    }

    private void slideToChangePosition(float deltaX) {
        deltaX = -deltaX;
        int width = getMeasuredWidth();
        int duration = mMediaController.getDuration();
        int currentPosition = mMediaController.getCurrentPosition();
        int position = (int) (deltaX / width * 120000 + currentPosition);
        if (position > duration) {
            position = duration;
        }
        if (position < 0) {
            position = 0;
        }
        mSeekPosition = position;
        if (duration > 0) {
            pbPercent.setProgress(100 * position / duration);
            tvPercent.setText(
                    String.format("%s/%s", CommonUtils.stringForTime(position),
                            CommonUtils.stringForTime(duration)));
        }
        if (mSeekPosition > currentPosition) {
            ivIcon.setImageDrawable(getResources().getDrawable(R.drawable.btn_ff_normal));
        } else {
            ivIcon.setImageDrawable(getResources().getDrawable(R.drawable.btn_fr_normal));
        }
        ivIcon.setVisibility(VISIBLE);
//        Log.v(TAG, " duration: " + duration + ", currentPosition: " + currentPosition + ",  seekPosition: " + mSeekPosition);
    }

    private void longPressToFFAndFR(MotionEvent e) {
        container.setVisibility(VISIBLE);
        int halfScreen = CommonUtils.getScreenWidth(mContext, false) / 2;
        if (e.getX() > halfScreen) {
            mCurSpeed = mMediaController.getSpeed();
            handler.sendEmptyMessage(ACTION_FAST_FORWARD);
            ivIcon.setImageDrawable(getResources().getDrawable(R.drawable.btn_ff_normal));
            ivIcon.setVisibility(VISIBLE);
        } else {
            handler.sendEmptyMessage(ACTION_FAST_REWARD);
            ivIcon.setImageDrawable(getResources().getDrawable(R.drawable.btn_fr_normal));
            ivIcon.setVisibility(VISIBLE);
        }
    }


    @Override
    public boolean onTouch(View v, MotionEvent event) {
        if (mMediaController.isLocked()) {
            return true;
        }
        int pointerCount = event.getPointerCount();
        if (!bHandlingOneTouch) {
            switch (event.getAction() & MotionEvent.ACTION_MASK) {
                case MotionEvent.ACTION_UP:
                    if (pointerCount == 2) {
                        mDownX1 = 0;
                        mDownY1 = 0;
                        mDownX2 = 0;
                        mDownY2 = 0;
                        bHandlingTwoTouch = true;
                        return true;
                    }
                    break;
                case MotionEvent.ACTION_MOVE:
                    if (pointerCount == 2) {
                        float x1 = event.getX(0);
                        float x2 = event.getX(1);
                        float y1 = event.getY(0);
                        float y2 = event.getY(1);

                        double changeX1 = x1 - mDownX1;
                        double changeX2 = x2 - mDownX2;
                        double changeY1 = y1 - mDownY1;
                        double changeY2 = y2 - mDownY2;

                        if (mMediaController.getVideoView().getScaleX() > 1) { // 滑动
                            float lessX = (float) ((changeX1) / 2 + (changeX2) / 2);
                            float lessY = (float) ((changeY1) / 2 + (changeY2) / 2);
                            setSelfPivot(-lessX, -lessY);
                        }
                        // 缩放处理
                        mMoveDist = spacing(event);
                        double space = mMoveDist - mOldDist;
                        float scale = (float) (mMediaController.getVideoView().getScaleX() + space / v.getWidth());
                        setScale(Math.max(SCALE_MIN, Math.min(SCALE_MAX, scale)));
                        bHandlingTwoTouch = true;
                        return true;
                    }
                    break;
                case MotionEvent.ACTION_POINTER_DOWN:
                    if (pointerCount == 2) {
                        mDownX1 = event.getX(0);
                        mDownX2 = event.getX(1);
                        mDownY1 = event.getY(0);
                        mDownY2 = event.getY(1);
//                        Log.v(TAG, "ACTION_POINTER_DOWN 双指按下 downX1=" + downX1 + " downX2="
//                                + downX2 + "  downY1=" + downY1 + " downY2=" + downY2);
                        mOldDist = spacing(event); // 两点按下时的距离
                        bHandlingTwoTouch = true;
                        return true;
                    }
                    break;
                case MotionEvent.ACTION_POINTER_UP:
                    if (pointerCount == 2) {
                        setInitScale();
                        bHandlingTwoTouch = false;
                        return true;
                    }
                    break;
                default:
                    break;
            }
        }
        return mGestureDetector.onTouchEvent(event);
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (bHandlingTwoTouch) {
            return true;
        }
        if (!mGestureDetector.onTouchEvent(event)) {
            int action = event.getAction();
            switch (action) {
                case MotionEvent.ACTION_UP:
                    bHandlingOneTouch = false;
                    bHandlingLongPress = false;
                    if (mSeekPosition >= 0) {
                        mMediaController.seek(mSeekPosition);
                        mSeekPosition = -1;
                    }
                    mMediaController.setSpeed(mCurSpeed);
                    handler.removeMessages(ACTION_FAST_FORWARD);
                    handler.removeMessages(ACTION_FAST_REWARD);
                    handler.sendEmptyMessageDelayed(ACTION_FADE_DIALOG, DIALOG_SHOW_TIME);
                    break;
                case MotionEvent.ACTION_CANCEL:
                    Log.w(TAG, "onTouchEvent Cancel, wont do anything");
                    bHandlingOneTouch = false;
                    bHandlingLongPress = false;
                    mSeekPosition = -1;
                    handler.removeMessages(ACTION_FAST_FORWARD);
                    handler.removeMessages(ACTION_FAST_REWARD);
                    handler.sendEmptyMessageDelayed(ACTION_FADE_DIALOG, DIALOG_SHOW_TIME);
                    break;
                default:
                    break;
            }
        }
        return super.onTouchEvent(event);
    }

    private void setSelfPivot(float lessX, float lessY) {
        float setPivotX = 0;
        float setPivotY = 0;
        setPivotX = getPivotX() + lessX;
        setPivotY = getPivotY() + lessY;
        if (setPivotX < 0 && setPivotY < 0) {
            setPivotX = 0;
            setPivotY = 0;
        } else if (setPivotX > 0 && setPivotY < 0) {
            setPivotY = 0;
            if (setPivotX > getWidth()) {
                setPivotX = getWidth();
            }
        } else if (setPivotX < 0 && setPivotY > 0) {
            setPivotX = 0;
            if (setPivotY > getHeight()) {
                setPivotY = getHeight();
            }
        } else {
            if (setPivotX > getWidth()) {
                setPivotX = getWidth();
            }
            if (setPivotY > getHeight()) {
                setPivotY = getHeight();
            }
        }
        setPivot(setPivotX, setPivotY);
    }

    /**
     * 计算两个点的距离
     */
    private double spacing(MotionEvent event) {
        if (event.getPointerCount() == 2) {
            float x = event.getX(0) - event.getX(1);
            float y = event.getY(0) - event.getY(1);
            return Math.sqrt(x * x + y * y);
        } else {
            return 0;
        }
    }

    /**
     * 平移画面，当画面的宽或高大于屏幕宽高时，调用此方法进行平移
     */
    private void setPivot(float x, float y) {
        mMediaController.getVideoView().setPivotX(x);
        mMediaController.getVideoView().setPivotY(y);
    }


    /**
     * 设置放大缩小
     */
    private void setScale(float scale) {
        mMediaController.getVideoView().setScaleX(scale);
        mMediaController.getVideoView().setScaleY(scale);
    }

    /**
     * 初始化比例
     */
    private void setInitScale() {
        mMediaController.getVideoView().setScaleX(1.0f);
        mMediaController.getVideoView().setScaleY(1.0f);
        setPivot(getWidth() / 2, getHeight() / 2);
    }
}
