package com.baidu.cloud.videoplayer.widget;

import java.lang.ref.WeakReference;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import com.baidu.cloud.media.player.IMediaPlayer;
import com.baidu.cloud.media.player.render.record.IRecordListener;
import com.baidu.cloud.videoplayer.demo.BaseApp;
import com.baidu.cloud.vlkit.VirtualLiveKit;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.SurfaceTexture;
import android.opengl.GLES30;
import android.opengl.GLSurfaceView;
import android.util.AttributeSet;
import android.util.Log;
import android.view.Surface;
import android.view.View;

public class VirtualLiveRenderView extends GLSurfaceView implements IRenderView {

    private static final String TAG = "VirtualLiveRenderView";
    private MeasureHelper mMeasureHelper;
    private GLVideoRender mVideoRender;
    private Context mContext;

    public VirtualLiveRenderView(Context context) {
        super(context);
        initView(context);
    }

    public VirtualLiveRenderView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initView(context);
    }

    private void initView(Context context) {
        mContext = context;
        mMeasureHelper = new MeasureHelper(this);
        setEGLContextClientVersion(3);
        mVideoRender = new GLVideoRender(context, this);
        setRenderer(mVideoRender);

        if (mVideoRender.mVirtualLiveKit == null) {
            mVideoRender.mVirtualLiveKit = new VirtualLiveKit();
            mVideoRender.mVirtualLiveKit.init(context, BaseApp.APP_ID);
        }
    }

    @Override
    public View getView() {
        return this;
    }

    @Override
    public boolean shouldWaitForResize() {
        return false;
    }

    @Override
    public void setVideoSize(int videoWidth, int videoHeight) {
        if (videoWidth > 0 && videoHeight > 0) {
            mMeasureHelper.setVideoSize(videoWidth, videoHeight);
            requestLayout();
        }
    }

    @Override
    public void setVideoSampleAspectRatio(int videoSarNum, int videoSarDen) {
        if (videoSarNum > 0 && videoSarDen > 0) {
            mMeasureHelper.setVideoSampleAspectRatio(videoSarNum, videoSarDen);
            requestLayout();
        }
    }

    @Override
    public void setVideoRotation(int degree) {
        mMeasureHelper.setVideoRotation(degree);
        setRotation(degree);
    }

    @Override
    public void setAspectRatio(int aspectRatio) {
        mMeasureHelper.setAspectRatio(aspectRatio);
        requestLayout();
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        mMeasureHelper.doMeasure(widthMeasureSpec, heightMeasureSpec);
        setMeasuredDimension(mMeasureHelper.getMeasuredWidth(), mMeasureHelper.getMeasuredHeight());
    }

    @Override
    public void addRenderCallback(IRenderView.IRenderCallback callback) {
        mVideoRender.addRenderCallback(callback);
    }

    @Override
    public void removeRenderCallback(IRenderView.IRenderCallback callback) {
        mVideoRender.removeRenderCallback(callback);
    }

    @Override
    public void release() {
        mVideoRender.release();
    }

    @Override
    public void destroy() {

    }

    @Override
    public Bitmap getBitmap() {
        return null;
    }

    @Override
    public void startRecord(String filePath) {
        // not support
    }

    @Override
    public void setRecordListener(IRecordListener listener) {
        // not support
    }

    @Override
    public void stopRecord() {
        // not support
    }

    @Override
    public void onUpdateAudioData(byte[] bytes, int byteSize) {
        // not support
    }


    private int mMediaPlayerCode = 0;

    public void setCurrentMediaPlayerCode(int currentCode) {
        mMediaPlayerCode = currentCode;
    }

    public int getCurrentMediaPlayerCode() {
        return mMediaPlayerCode;
    }

    private SurfaceTexture lastSurfaceTexture;

    public SurfaceTexture getLastSurfaceTexture() {
        return lastSurfaceTexture;
    }

    public void setLastSurfaceTexture(SurfaceTexture surfaceTexture) {
        lastSurfaceTexture = surfaceTexture;
    }

    public IRenderView.ISurfaceHolder getSurfaceHolder() {
        return new VirtualLiveRenderView.InternalSurfaceHolder(this);
    }

    private static final class InternalSurfaceHolder implements IRenderView.ISurfaceHolder {
        private VirtualLiveRenderView mVirtualLiveSurfaceRenderView;

        public InternalSurfaceHolder(VirtualLiveRenderView virtualLiveRenderView) {
            mVirtualLiveSurfaceRenderView = virtualLiveRenderView;
        }

        public void bindToMediaPlayer(IMediaPlayer mp) {
            if (mp == null
                    || mVirtualLiveSurfaceRenderView.mVideoRender.getSurfaceTexture() == null) {
                // if reSetRender is invoked twice during one playing, surfaceTexture maybe null
                return;
            }
            // Tricky: 由 Object 类定义的 hashCode 方法确实会针对不同的对象返回不同的整数。
            // （这一般是通过将该对象的内部地址转换成一个整数来实现的，但是 Java语言不需要这种实现技巧。）
            if (mp.hashCode() != mVirtualLiveSurfaceRenderView.getCurrentMediaPlayerCode()) {
                // new player
                mp.setSurface(openSurface());
            } else {
                // old player
                if (!mVirtualLiveSurfaceRenderView.getLastSurfaceTexture()
                        .equals(mVirtualLiveSurfaceRenderView.mVideoRender.getSurfaceTexture())) {
                    mVirtualLiveSurfaceRenderView.mVideoRender.
                            setSurfaceTexture(mVirtualLiveSurfaceRenderView.getLastSurfaceTexture());
                }
            }
            mVirtualLiveSurfaceRenderView.setCurrentMediaPlayerCode(mp.hashCode());
        }

        @Override
        public IRenderView getRenderView() {
            return mVirtualLiveSurfaceRenderView;
        }

        @Override
        public Surface openSurface() {
            return new Surface(mVirtualLiveSurfaceRenderView.mVideoRender.getSurfaceTexture());
        }
    }

    public Surface openBgSurface() {
        return new Surface(mVideoRender.getBgSurfaceTexture());
    }

    private static class GLVideoRender
            implements Renderer, SurfaceTexture.OnFrameAvailableListener {

        private GLTextureRender mFgGLTextureRender;
        private GLTextureRender mBgGLTextureRender;
        private SurfaceTexture mSurfaceTexture;
        private SurfaceTexture mBgSurfaceTexture;
        private boolean updateSurface = false;
        private boolean updateBgSurface = false;
        private int mFboId = -1;
        private int mFgInputTex = -1;
        private int mFgOutputTex = -1;
        private int mBgInputTex = -1;
        private VirtualLiveKit mVirtualLiveKit;

        private WeakReference<VirtualLiveRenderView> mWeakRenderView;
        private Map<IRenderView.IRenderCallback, Object>
                mRenderCallbackMap = new ConcurrentHashMap<>();


        private int width;
        private int height;

        public GLVideoRender(Context context, VirtualLiveRenderView renderView) {
            mFgGLTextureRender = new GLTextureRender();
            mBgGLTextureRender = new GLTextureRender();
            mWeakRenderView = new WeakReference<>(renderView);
        }

        public SurfaceTexture getSurfaceTexture() {
            return mSurfaceTexture;
        }

        public SurfaceTexture getBgSurfaceTexture() {
            return mBgSurfaceTexture;
        }

        public void setSurfaceTexture(SurfaceTexture surfaceTexture) {
            if (surfaceTexture.equals(mSurfaceTexture)) {
                return;
            }
            if (mSurfaceTexture != null) {
                mSurfaceTexture.release();
            }
            mSurfaceTexture = surfaceTexture;
            mSurfaceTexture.setOnFrameAvailableListener(this);

            synchronized(this) {
                updateSurface = false;
            }
        }

        public void addRenderCallback(IRenderView.IRenderCallback callback) {
            mRenderCallbackMap.put(callback, callback);

            IRenderView.ISurfaceHolder surfaceHolder = null;
            if (mSurfaceTexture != null) {
                surfaceHolder =
                        new VirtualLiveRenderView.InternalSurfaceHolder(mWeakRenderView.get());
                callback.onSurfaceCreated(surfaceHolder, width, height);
            }

            if (width > 0 && height > 0) {
                if (surfaceHolder == null) {
                    surfaceHolder =
                            new VirtualLiveRenderView.InternalSurfaceHolder(mWeakRenderView.get());
                }
                callback.onSurfaceChanged(surfaceHolder, 0, width, height);
            }
        }

        public void removeRenderCallback(IRenderView.IRenderCallback callback) {
            mRenderCallbackMap.remove(callback);
        }

        @Override
        public void onDrawFrame(GL10 glUnused) {
            // 在单独的线程
            synchronized(this) {
                if (updateSurface) {
                    mSurfaceTexture.updateTexImage();
                    updateSurface = false;
                }
                if (updateBgSurface) {
                    mBgSurfaceTexture.updateTexImage();
                    updateBgSurface = false;
                }
            }
            if (true) {
                synchronized (this) {
                    if (mVirtualLiveKit != null) {
                        GLTextureRender.bindTextureToFBO(mFgInputTex, GLES30.GL_TEXTURE_2D, mFboId);
                        mFgGLTextureRender.drawFrame(mSurfaceTexture, mFboId);

                        mVirtualLiveKit.drawFrameFromTexId();

                        mBgGLTextureRender.mixSurfaceAndTex(mBgSurfaceTexture, mFgOutputTex, 0);
                    } else {
                        mFgGLTextureRender.drawFrame(mSurfaceTexture, 0);
                    }
                }
            } else {
                mFgGLTextureRender.drawFrame(mSurfaceTexture, 0);
            }
        }

        @Override
        public void onSurfaceChanged(GL10 glUnused, int width, int height) {
            if (mFgInputTex != -1) {
                GLTextureRender.destroyTextureObject(mFgInputTex);
            }
            mFgInputTex = GLTextureRender.createTextureObject(GLES30.GL_TEXTURE_2D, width, height);

            if (mBgInputTex != -1) {
                GLTextureRender.destroyTextureObject(mBgInputTex);
            }
            mBgInputTex = GLTextureRender.createTextureObject(GLES30.GL_TEXTURE_2D, width, height);

            if (mFgOutputTex != -1) {
                GLTextureRender.destroyTextureObject(mFgOutputTex);
            }
            mFgOutputTex = GLTextureRender.createTextureObject(GLES30.GL_TEXTURE_2D, width, height);

            mFgGLTextureRender.surfaceChanged(width, height);
            mBgGLTextureRender.surfaceChanged(width, height);
            this.width = width;
            this.height = height;

            mVirtualLiveKit.prepareWithInputTexId(mFgInputTex);
            mVirtualLiveKit.setOutputTexId(mFgOutputTex);
            mVirtualLiveKit.setInputSize(width, height);
            mVirtualLiveKit.setOutputSize(width, height);

            IRenderView.ISurfaceHolder
                    surfaceHolder =
                    new VirtualLiveRenderView.InternalSurfaceHolder(mWeakRenderView.get());
            for (IRenderView.IRenderCallback renderCallback : mRenderCallbackMap.keySet()) {
                renderCallback.onSurfaceChanged(surfaceHolder, 0, width, height);
            }
        }

        @Override
        public void onSurfaceCreated(GL10 gl, EGLConfig config) {
            mFboId = GLTextureRender.createFrameBufferObject();

            mFgGLTextureRender.surfaceCreated();
            mBgGLTextureRender.surfaceCreated();
            /*
             * Create the SurfaceTexture that will feed this textureID,
             * and pass it to the MediaPlayer
             */
            mSurfaceTexture = new SurfaceTexture(mFgGLTextureRender.getTextureId());
            mSurfaceTexture.setOnFrameAvailableListener(this);

            mBgSurfaceTexture = new SurfaceTexture(mBgGLTextureRender.getTextureId());
            mBgSurfaceTexture.setOnFrameAvailableListener(this);
            
            if (mWeakRenderView.get() == null) {
                // should never come in
                Log.e(TAG, "!!!!!Too bad, textureview in callback is released. function will not work normally");
            } else {
                // only set once
                if (mWeakRenderView.get().getLastSurfaceTexture() == null) {
                    mWeakRenderView.get().setLastSurfaceTexture(mSurfaceTexture);
                }
            }

            width = 0;
            height = 0;

            IRenderView.ISurfaceHolder surfaceHolder =
                    new VirtualLiveRenderView.InternalSurfaceHolder(mWeakRenderView.get());
            for (IRenderView.IRenderCallback renderCallback : mRenderCallbackMap.keySet()) {
                renderCallback.onSurfaceCreated(surfaceHolder, 0, 0);
            }

            synchronized(this) {
                updateSurface = false;
                updateBgSurface = false;
            }
        }

        @Override
        public synchronized void onFrameAvailable(SurfaceTexture surface) {
            // 在主线程
            if (surface.equals(mSurfaceTexture)) {
                updateSurface = true;
            }
            if (surface.equals(mBgSurfaceTexture)) {
                updateBgSurface = true;
            }
        }

        public void release() {
            width = 0;
            height = 0;

            IRenderView.ISurfaceHolder surfaceHolder =
                    new VirtualLiveRenderView.InternalSurfaceHolder(mWeakRenderView.get());
            for (IRenderView.IRenderCallback renderCallback : mRenderCallbackMap.keySet()) {
                renderCallback.onSurfaceDestroyed(surfaceHolder);
            }

            synchronized(this) {
                if (mSurfaceTexture != null) {
                    mSurfaceTexture.setOnFrameAvailableListener(null);
                    mSurfaceTexture.release();
                }
                if (mBgSurfaceTexture != null) {
                    mBgSurfaceTexture.setOnFrameAvailableListener(null);
                    mBgSurfaceTexture.release();
                }
                updateSurface = false;
                updateBgSurface = false;

                if (mVirtualLiveKit != null) {
                    mVirtualLiveKit.release();
                    mVirtualLiveKit = null;
                }
            }
        }
    }
}